#ifndef __ASM_MACH_LOONGSON64_IRQ_H_
#define __ASM_MACH_LOONGSON64_IRQ_H_

#include <boot_param.h>

#ifdef CONFIG_CPU_LOONGSON3

#define NR_IRQS	256
/* cpu core interrupt numbers */
#define MIPS_CPU_IRQ_BASE 56

#define LOONGSON_UART_IRQ   (MIPS_CPU_IRQ_BASE + 2) /* UART */
#define LOONGSON_BRIDGE_IRQ (MIPS_CPU_IRQ_BASE + 3) /* CASCADE */
#define LOONGSON_TIMER_IRQ  (MIPS_CPU_IRQ_BASE + 7) /* CPU Timer */

#define LOONGSON_HT1_CFG_BASE		loongson_sysconf.ht_control_base
#define LOONGSON_HT1_INT_VECTOR_BASE	(LOONGSON_HT1_CFG_BASE + 0x80)
#define LOONGSON_HT1_INT_EN_BASE	(LOONGSON_HT1_CFG_BASE + 0xa0)
#define LOONGSON_HT1_INT_VECTOR(n)	\
		LOONGSON3_REG32(LOONGSON_HT1_INT_VECTOR_BASE, 4 * (n))
#define LOONGSON_HT1_INTN_EN(n)		\
		LOONGSON3_REG32(LOONGSON_HT1_INT_EN_BASE, 4 * (n))

#define LOONGSON_INT_ROUTER_OFFSET	0x1400
#define LOONGSON_INT_ROUTER_INTEN	\
	  LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x24)
#define LOONGSON_INT_ROUTER_INTENSET	\
	  LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x28)
#define LOONGSON_INT_ROUTER_INTENCLR	\
	  LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x2c)
#define LOONGSON_INT_ROUTER_ENTRY(n)	\
	  LOONGSON3_REG8(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + n)
#define LOONGSON_INT_ROUTER_LPC		LOONGSON_INT_ROUTER_ENTRY(0x0a)
#define LOONGSON_INT_ROUTER_HT1(n)	LOONGSON_INT_ROUTER_ENTRY(n + 0x18)

#define LOONGSON_INT_COREx_INTy(x, y)	(1<<(x) | 1<<(y+4))	/* route to int y of core x */

#define LS2H_PCH_IRQ_BASE		64

/* group 0 */
#define LS2H_PCH_ACPI_IRQ		(0 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HPET_IRQ		(1 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_UART0_IRQ		(2 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_UART1_IRQ		(3 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_UART2_IRQ		(4 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_UART3_IRQ		(5 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_SPI_IRQ		(6 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_I2C0_IRQ		(7 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_I2C1_IRQ		(8 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_AC97_IRQ		(9 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_DMA0_IRQ		(10 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_DMA1_IRQ		(11 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_DMA2_IRQ		(12 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_LPC_IRQ		(13 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_RTC_INT0_IRQ		(14 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_RTC_INT1_IRQ		(15 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_RTC_INT2_IRQ		(16 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_TOY_INT0_IRQ		(17 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_TOY_INT1_IRQ		(18 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_TOY_INT2_IRQ		(19 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_RTC_TICK_IRQ		(20 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_TOY_TICK_IRQ		(21 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_NAND_IRQ		(22 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_SYS_INTN_IRQ		(23 + LS2H_PCH_IRQ_BASE)

/* group 1 */
#define LS2H_PCH_EHCI_IRQ		(32 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_OHCI_IRQ		(33 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_OTG_IRQ		(34 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GMAC0_IRQ		(35 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GMAC1_IRQ		(36 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_SATA_IRQ		(37 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPU_IRQ		(38 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_DC_IRQ			(39 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PWM0_IRQ		(40 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PWM1_IRQ		(41 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PWM2_IRQ		(42 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PWM3_IRQ		(43 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT0_IRQ		(44 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT1_IRQ		(45 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT2_IRQ		(46 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT3_IRQ		(47 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT4_IRQ		(48 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT5_IRQ		(49 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT6_IRQ		(50 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT7_IRQ		(51 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PCIE_PORT0_IRQ		(52 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PCIE_PORT1_IRQ		(53 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PCIE_PORT2_IRQ		(54 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PCIE_PORT3_IRQ		(55 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_SATA_PHY_IRQ		(56 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HDA_IRQ		(57 + LS2H_PCH_IRQ_BASE)

#define LS2H_PCIE_PORT0_INTA_BIT	(1 << 20)
#define LS2H_PCIE_PORT1_INTA_BIT	(1 << 21)
#define LS2H_PCIE_PORT2_INTA_BIT	(1 << 22)
#define LS2H_PCIE_PORT3_INTA_BIT	(1 << 23)

/* group 2 */
#define LS2H_PCH_GPIO0_IRQ		(64 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO1_IRQ		(65 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO2_IRQ		(66 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO3_IRQ		(67 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO4_IRQ		(68 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO5_IRQ		(69 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO6_IRQ		(70 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO7_IRQ		(71 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO8_IRQ		(72 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO9_IRQ		(73 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO10_IRQ		(74 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO11_IRQ		(75 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO12_IRQ		(76 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO13_IRQ		(77 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO14_IRQ		(78 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO15_IRQ		(79 + LS2H_PCH_IRQ_BASE)

#define LS2H_PCH_GPIO_IRQ		LS2H_PCH_GPIO0_IRQ
#define LS2H_PCH_GPIO_FIRST_IRQ		LS2H_PCH_GPIO0_IRQ
#define LS2H_PCH_GPIO_IRQ_COUNT		96
#define LS2H_PCH_GPIO_LAST_IRQ		(LS2H_PCH_GPIO_FIRST_IRQ + LS2H_PCH_GPIO_IRQ_COUNT - 1)

#define LS2H_PCH_LAST_IRQ		(160 + LS2H_PCH_IRQ_BASE)

#define LS2H_PCH_KBD_IRQ		1
#define LS2H_PCH_AUX_IRQ		12

struct ls2h_intctl_regs {
	volatile u32 int_isr;
	volatile u32 int_en;
	volatile u32 int_set;
	volatile u32 int_clr;
	volatile u32 int_pol;
	volatile u32 int_edge;
};
#endif

extern void fixup_irqs(void);
extern void loongson3_ipi_interrupt(struct pt_regs *regs);

#include_next <irq.h>
#endif /* __ASM_MACH_LOONGSON64_IRQ_H_ */
